@@ -22,12 +22,11 @@
 
 #include <LYCookie.h>
 #include <LYCharSets.h>
+#include <LYCharUtils.h>
 #include <LYStrings.h>
 #include <LYUtils.h>
 #include <LYLeaks.h>
 
-extern BOOL HTPassEightBitRaw;
-
 /*		MIME Object
 **		-----------
 */
@@ -75,6 +74,9 @@ typedef enum {
 	miPRAGMA,
 	miPROXY_AUTHENTICATE,
 	miPUBLIC,
+	miR,
+	miRE,
+	miREFRESH,
 	miRETRY_AFTER,
 	miS,
 	miSAFE,
@@ -130,6 +132,8 @@ struct _HTStream {
 	char *			set_cookie2;	/* Set-Cookie2 */
 	char *			location;	/* Location */
 
+	char *			refresh_url;	/* "Refresh:" URL */
+
 	HTFormat		encoding;	/* Content-Transfer-Encoding */
 	char *			compression_encoding;
 	HTFormat		format;		/* Content-Type */
@@ -165,6 +169,32 @@ PUBLIC void HTMIME_TrimDoubleQuotes ARGS1(
 	value[i] = cp[(i +1)];
 }
 
+PRIVATE BOOL content_is_compressed ARGS1(HTStream *, me)
+{
+    char *encoding = me->anchor->content_encoding;
+
+    return encoding != 0
+        && strcmp(encoding, "8bit") != 0
+	&& strcmp(encoding, "7bit") != 0
+	&& strcmp(encoding, "binary") != 0;
+}
+
+/*
+ * Strip quotes from a refresh-URL.
+ */
+PRIVATE void dequote ARGS1(char *, url)
+{
+    int len;
+
+    len = strlen(url);
+    if (*url == '\'' && len > 1 && url[len-1] == url[0]) {
+	url[len-1] = '\0';
+	while ((url[0] = url[1]) != '\0') {
+	    ++url;
+	}
+    }
+}
+
 PRIVATE int pumpData ARGS1(HTStream *, me)
 {
     if (strchr(HTAtom_name(me->format), ';') != NULL) {
@@ -174,13 +204,13 @@ PRIVATE int pumpData ARGS1(HTStream *, me)
 		HTAtom_name(me->format)));
 	StrAllocCopy(cp, HTAtom_name(me->format));
 	/*
-	**	Note that the Content-Type value was converted
-	**	to lower case when we loaded into me->format,
-	**	but there may have been a mixed or upper-case
-	**	atom, so we'll force lower-casing again.  We
-	**	also stripped spaces and double-quotes, but
-	**	we'll make sure they're still gone from any
-	**	charset parameter we check. - FM
+	** Note that the Content-Type value was converted
+	** to lower case when we loaded into me->format,
+	** but there may have been a mixed or upper-case
+	** atom, so we'll force lower-casing again.  We
+	** also stripped spaces and double-quotes, but
+	** we'll make sure they're still gone from any
+	** charset parameter we check.  - FM
 	*/
 	LYLowerCase(cp);
 	if ((cp1 = strchr(cp, ';')) != NULL) {
@@ -352,7 +382,7 @@ PRIVATE int pumpData ARGS1(HTStream *, me)
 		LYmktime(me->anchor->date, TRUE)) {
 		me->anchor->no_cache = TRUE;
 	    }
-	} else if (LYmktime(me->anchor->expires, FALSE) <= 0) {
+	} else if (LYmktime(me->anchor->expires, FALSE) == 0) {
 	    /*
 	    **  We don't have a Date header, and
 	    **  the value is in past for us. - FM
@@ -432,6 +462,26 @@ PRIVATE int pumpData ARGS1(HTStream *, me)
     } else {
 	me->state = MIME_IGNORE;	/* What else to do? */
     }
+    if (me->refresh_url != NULL && !content_is_compressed(me)) {
+	char *url = NULL;
+	char *num = NULL;
+	char *txt = NULL;
+	char *base = "";	/* FIXME: refresh_url may be relative to doc */
+
+	LYParseRefreshURL(me->refresh_url, &num, &url);
+	if (url != NULL && me->format == WWW_HTML) {
+	    CTRACE((tfp, "Formatting refresh-url as first line of result\n"));
+	    HTSprintf0(&txt, gettext("Refresh: "));
+	    HTSprintf(&txt, gettext("%s seconds "), num);
+	    dequote(url);
+	    HTSprintf(&txt, "<a href=\"%s%s\">%s</a><br>", base, url, url);
+	    CTRACE((tfp, "URL %s%s\n", base, url));
+	    (me->isa->put_string)(me, txt);
+	    free(txt);
+	}
+	FREE(num);
+	FREE(url);
+    }
     return HT_OK;
 }
 
@@ -605,20 +655,18 @@ PRIVATE int dispatchField ARGS1(HTStream *, me)
 	LYLowerCase(me->value);
 	StrAllocCopy(me->anchor->content_encoding, me->value);
 	FREE(me->compression_encoding);
-	if (!strcmp(me->value, "8bit") ||
-	    !strcmp(me->value, "7bit") ||
-	    !strcmp(me->value, "binary")) {
-	    /*
-	    **	Some server indicated "8bit", "7bit" or "binary"
-	    **	inappropriately.  We'll ignore it. - FM
-	    */
-	    CTRACE((tfp, "                Ignoring it!\n"));
-	} else {
+	if (content_is_compressed(me)) {
 	    /*
 	    **	Save it to use as a flag for setting
 	    **	up a "www/compressed" target. - FM
 	    */
 	    StrAllocCopy(me->compression_encoding, me->value);
+	} else {
+	    /*
+	    **	Some server indicated "8bit", "7bit" or "binary"
+	    **	inappropriately.  We'll ignore it. - FM
+	    */
+	    CTRACE((tfp, "                Ignoring it!\n"));
 	}
 	break;
     case miCONTENT_FEATURES:
@@ -800,6 +848,12 @@ PRIVATE int dispatchField ARGS1(HTStream *, me)
 	CTRACE((tfp, "HTMIME: PICKED UP Public: '%s'\n",
 		me->value));
 	break;
+    case miREFRESH:		/* nonstandard: Netscape */
+	HTMIME_TrimDoubleQuotes(me->value);
+	CTRACE((tfp, "HTMIME: PICKED UP Refresh: '%s'\n",
+		me->value));
+	StrAllocCopy(me->refresh_url, me->value);
+	break;
     case miRETRY_AFTER:
 	HTMIME_TrimDoubleQuotes(me->value);
 	CTRACE((tfp, "HTMIME: PICKED UP Retry-After: '%s'\n",
@@ -1032,10 +1086,8 @@ PRIVATE void HTMIME_put_character ARGS2(
 
 	case 'r':
 	case 'R':
-	    me->check_pointer = "etry-after:";
-	    me->if_ok = miRETRY_AFTER;
-	    me->state = miCHECK;
-	    CTRACE((tfp, "HTMIME: Got 'R' at beginning of line, checking for 'etry-after'\n"));
+	    me->state = miR;
+	    CTRACE((tfp, "HTMIME: Got 'R' at beginning of line, state now R\n"));
 	    break;
 
 	case 's':
@@ -1323,6 +1375,47 @@ PRIVATE void HTMIME_put_character ARGS2(
 	} /* switch on character */
 	break;
 
+    case miR:				/* Check for 'e' */
+	switch (c) {
+	case 'e':
+	case 'E':
+	    me->state = miRE;
+	    CTRACE((tfp, "HTMIME: Was R, found E\n"));
+	    break;
+	default:
+	    CTRACE((tfp, "HTMIME: Bad character `%c' found where `%s' expected\n",
+			c, "'e'"));
+	    goto bad_field_name;
+
+	} /* switch on character */
+	break;
+
+    case miRE:				/* Check for 'a' or 'o' */
+	switch (c) {
+	case 'f':
+	case 'F':			/* nonstandard: Netscape */
+	    me->check_pointer = "resh:";
+	    me->if_ok = miREFRESH;
+	    me->state = miCHECK;
+	    CTRACE((tfp, "HTMIME: Was RE, found F, checking for '%s'\n", me->check_pointer));
+	    break;
+
+	case 't':
+	case 'T':
+	    me->check_pointer = "ry-after:";
+	    me->if_ok = miRETRY_AFTER;
+	    me->state = miCHECK;
+	    CTRACE((tfp, "HTMIME: Was RE, found T, checking for '%s'\n", me->check_pointer));
+	    break;
+
+	default:
+	    CTRACE((tfp, "HTMIME: Bad character `%c' found where `%s' expected\n",
+			c, "'f' or 't'"));
+	    goto bad_field_name;
+
+	} /* switch on character */
+	break;
+
     case miS:				/* Check for 'a' or 'e' */
 	switch (c) {
 	case 'a':
@@ -1672,6 +1765,7 @@ PRIVATE void HTMIME_put_character ARGS2(
     case miPRAGMA:
     case miPROXY_AUTHENTICATE:
     case miPUBLIC:
+    case miREFRESH:
     case miRETRY_AFTER:
     case miSAFE:
     case miSERVER:
@@ -1889,8 +1983,9 @@ PUBLIC HTStream* HTMIMEConvert ARGS3(
     me->format	  =	WWW_HTML;
     me->targetRep =	pres->rep_out;
     me->boundary  =	NULL;		/* Not set yet */
-    me->set_cookie  =	NULL;		/* Not set yet */
-    me->set_cookie2  =	NULL;		/* Not set yet */
+    me->set_cookie =	NULL;		/* Not set yet */
+    me->set_cookie2 =	NULL;		/* Not set yet */
+    me->refresh_url =	NULL;		/* Not set yet */
     me->encoding  =	0;		/* Not set yet */
     me->compression_encoding = NULL;	/* Not set yet */
     me->net_ascii =	NO;		/* Local character set */
@@ -1967,27 +2062,23 @@ PUBLIC HTStream* HTMIMERedirect ARGS3(
 **
 **	Written by S. Ichikawa,
 **	partially inspired by encdec.c of <jh@efd.lth.se>.
-**	Assume caller's buffer is LINE_LENGTH bytes, these decode to
-**	no longer than the input strings.
 */
-#define LINE_LENGTH 512		/* Maximum length of line of ARTICLE etc */
-#ifdef ESC
-#undef ESC
-#endif /* ESC */
 #include <LYCharVals.h>  /* S/390 -- gil -- 0163 */
-#define ESC	CH_ESC
 
 PRIVATE char HTmm64[] =
     "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=" ;
 PRIVATE char HTmmquote[] = "0123456789ABCDEF";
 PRIVATE int HTmmcont = 0;
 
-PUBLIC void HTmmdec_base64 ARGS2(
-	char *,		t,
+PRIVATE void HTmmdec_base64 ARGS2(
+	char **,	t,
 	char *,		s)
 {
     int   d, count, j, val;
-    char  buf[LINE_LENGTH], *bp, nw[4], *p;
+    char *buf, *bp, nw[4], *p;
+
+    if ((buf = malloc(strlen(s) * 3 + 1)) == 0)
+	outofmem(__FILE__, "HTmmdec_base64");
 
     for (bp = buf; *s; s += 4) {
 	val = 0;
@@ -2018,14 +2109,18 @@ PUBLIC void HTmmdec_base64 ARGS2(
 	    *bp++ = nw[2];
     }
     *bp = '\0';
-    strcpy(t, buf);
+    StrAllocCopy(*t, buf);
+    FREE(buf);
 }
 
-PUBLIC void HTmmdec_quote ARGS2(
-	char *,		t,
+PRIVATE void HTmmdec_quote ARGS2(
+	char **,	t,
 	char *,		s)
 {
-    char  buf[LINE_LENGTH], cval, *bp, *p;
+    char *buf, cval, *bp, *p;
+
+    if ((buf = malloc(strlen(s) + 1)) == 0)
+	outofmem(__FILE__, "HTmmdec_quote");
 
     for (bp = buf; *s; ) {
 	if (*s == '=') {
@@ -2052,23 +2147,27 @@ PUBLIC void HTmmdec_quote ARGS2(
 	}
     }
     *bp = '\0';
-    strcpy(t, buf);
+    StrAllocCopy(*t, buf);
+    FREE(buf);
 }
 
 /*
 **	HTmmdecode for ISO-2022-JP - FM
 */
 PUBLIC void HTmmdecode ARGS2(
-	char *,		trg,
-	char *,		str)
+	char **,	target,
+	char *,		source)
 {
-    char buf[LINE_LENGTH], mmbuf[LINE_LENGTH];
+    char *buf;
+    char *mmbuf = NULL;
+    char *m2buf = NULL;
     char *s, *t, *u;
     int  base64, quote;
 
-    buf[0] = '\0';
-
-    for (s = str, u = buf; *s; ) {
+    if ((buf = malloc(strlen(source) + 1)) == 0)
+	outofmem(__FILE__, "HTmmdecode");
+  
+    for (s = source, u = buf; *s;) {
 	if (!strncasecomp(s, "=?ISO-2022-JP?B?", 16)) {
 	    base64 = 1;
 	} else {
@@ -2082,15 +2181,18 @@ PUBLIC void HTmmdecode ARGS2(
 	if (base64 || quote) {
 	    if (HTmmcont) {
 		for (t = s - 1;
-		    t >= str && (*t == ' ' || *t == '\t'); t--) {
+		    t >= source && (*t == ' ' || *t == '\t'); t--) {
 			u--;
 		}
 	    }
+	    if (mmbuf == 0)	/* allocate buffer big enough for source */
+		StrAllocCopy(mmbuf, source);
 	    for (s += 16, t = mmbuf; *s; ) {
 		if (s[0] == '?' && s[1] == '=') {
 		    break;
 		} else {
 		    *t++ = *s++;
+		    *t = '\0';
 		}
 	    }
 	    if (s[0] != '?' || s[1] != '=') {
@@ -2100,14 +2202,12 @@ PUBLIC void HTmmdecode ARGS2(
 		*t = '\0';
 	    }
 	    if (base64)
-		HTmmdec_base64(mmbuf, mmbuf);
+		HTmmdec_base64(&m2buf, mmbuf);
 	    if (quote)
-		HTmmdec_quote(mmbuf, mmbuf);
-	    for (t = mmbuf; *t; )
+		HTmmdec_quote(&m2buf, mmbuf);
+	    for (t = m2buf; *t; )
 		*u++ = *t++;
 	    HTmmcont = 1;
-	    /* if (*s == ' ' || *s == '\t') *u++ = *s; */
-	    /* for ( ; *s == ' ' || *s == '\t'; s++) ; */
 	} else {
 	    if (*s != ' ' && *s != '\t')
 		HTmmcont = 0;
@@ -2116,7 +2216,10 @@ PUBLIC void HTmmdecode ARGS2(
     }
     *u = '\0';
 end:
-    strcpy(trg, buf);
+    StrAllocCopy(*target, buf);
+    FREE(m2buf);
+    FREE(mmbuf);
+    FREE(buf);
 }
 
 /*
@@ -2124,22 +2227,27 @@ end:
 **  (The author of this function "rjis" is S. Ichikawa.)
 */
 PUBLIC int HTrjis ARGS2(
-	char *,		t,
+	char **,	t,
 	char *,		s)
 {
-    char *p, buf[LINE_LENGTH];
+    char *p;
+    char *buf = NULL;
     int kanji = 0;
 
-    if (strchr(s, ESC) || !strchr(s, '$')) {
-	if (s != t)
-	    strcpy(t, s);
+    if (strchr(s, CH_ESC) || !strchr(s, '$')) {
+	if (s != *t)
+	    StrAllocCopy(*t, s);
 	return 1;
     }
+
+    if ((buf = malloc(strlen(s) * 2 + 1)) == 0)
+	outofmem(__FILE__, "HTrjis");
+
     for (p = buf; *s; ) {
 	if (!kanji && s[0] == '$' && (s[1] == '@' || s[1] == 'B')) {
 	    if (HTmaybekanji((int)s[2], (int)s[3])) {
 		kanji = 1;
-		*p++ = ESC;
+		*p++ = CH_ESC;
 		*p++ = *s++;
 		*p++ = *s++;
 		*p++ = *s++;
@@ -2151,7 +2259,7 @@ PUBLIC int HTrjis ARGS2(
 	}
 	if (kanji && s[0] == '(' && (s[1] == 'J' || s[1] == 'B')) {
 	    kanji = 0;
-	    *p++ = ESC;
+	    *p++ = CH_ESC;
 	    *p++ = *s++;
 	    *p++ = *s++;
 	    continue;
@@ -2160,7 +2268,8 @@ PUBLIC int HTrjis ARGS2(
     }
     *p = *s;	/* terminate string */
 
-    strcpy(t, buf);
+    StrAllocCopy(*t, buf);
+    FREE(buf);
     return 0;
 }
 
@@ -2172,7 +2281,7 @@ PUBLIC int HTrjis ARGS2(
 */
 /*
  * RJIS ( Recover JIS code from broken file )
- * $Header: /home/cvs/cvsroot/src/lynx2-8-4/WWW/Library/Implementation/HTMIME.c,v 1.1.1.1 2001/08/08 07:47:06 corinna Exp $
+ * $Header: /home/cvs/cvsroot/src/lynx2-8-5/WWW/Library/Implementation/HTMIME.c,v 1.2 2005/11/15 10:24:56 corinna Exp $
  * Copyright (C) 1992 1994
  * Hironobu Takahashi (takahasi@tiny.or.jp)
  *
